#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2017/12/11 11:08
# @Author  : zhm
# @File    : LunarSolarConverter.py
# @Software: PyCharm
from pprint import pprint


class Lunar:
    def __init__(self, lunarYear, lunarMonth, lunarDay, isleap):
        self.isleap = isleap
        self.lunarDay = lunarDay
        self.lunarMonth = lunarMonth
        self.lunarYear = lunarYear


class Solar:
    def __init__(self, solarYear, solarMonth, solarDay):
        self.solarDay = solarDay
        self.solarMonth = solarMonth
        self.solarYear = solarYear


def GetBitInt(data, length, shift):
    return (data & (((1 << length) - 1) << shift)) >> shift


def SolarToInt(y, m, d):
    m = (m + 9) % 12
    y -= int(m / 10)
    return 365 * y + int(y / 4) - int(y / 100) + int(y / 400) + int((m * 306 + 5) / 10) + (d - 1)


def SolarFromInt(g):
    y = int((10000 * g + 14780) / 3652425)
    ddd = g - (365 * y + int(y / 4) - int(y / 100) + int(y / 400))
    if ddd < 0:
        y -= 1
        ddd = g - (365 * y + int(y / 4) - int(y / 100) + int(y / 400))

    mi = int((100 * ddd + 52) / 3060)
    mm = (mi + 2) % 12 + 1
    y += int((mi + 2) / 12)
    dd = ddd - int((mi * 306 + 5) / 10) + 1
    solar = Solar(y, mm, dd)
    return solar


class LunarSolarConverter:
    #####################################################################################
    # 1888~2111年农历数据表
    # 农历数据 每个元素的存储格式如下：
    #   16~13    12          11~0
    #  闰几月 闰月日数  1~12月份农历日数(大小月)
    # 注：1、bit0表示农历1月份日数，为1表示30天，为0表示29天。bit1表示农历2月份日数，依次类推。
    #     2、bit12表示闰月日数，1为30天，0为29天。bit16~bit13表示第几月是闰月(注：为0表示该年无闰月)
    # 数据来源参考: http://data.weather.gov.hk/gts/time/conversion1_text_c.htm
    #####################################################################################
    lunar_month_days = [1887, 0x1694, 0x16aa, 0x4ad5, 0xab6, 0xc4b7, 0x4ae, 0xa56, 0xb52a,
                        0x1d2a, 0xd54, 0x75aa, 0x156a, 0x1096d, 0x95c, 0x14ae, 0xaa4d, 0x1a4c, 0x1b2a, 0x8d55,
                        0xad4, 0x135a, 0x495d,
                        0x95c, 0xd49b, 0x149a, 0x1a4a, 0xbaa5, 0x16a8, 0x1ad4, 0x52da, 0x12b6, 0xe937, 0x92e,
                        0x1496, 0xb64b, 0xd4a,
                        0xda8, 0x95b5, 0x56c, 0x12ae, 0x492f, 0x92e, 0xcc96, 0x1a94, 0x1d4a, 0xada9, 0xb5a, 0x56c,
                        0x726e, 0x125c,
                        0xf92d, 0x192a, 0x1a94, 0xdb4a, 0x16aa, 0xad4, 0x955b, 0x4ba, 0x125a, 0x592b, 0x152a,
                        0xf695, 0xd94, 0x16aa,
                        0xaab5, 0x9b4, 0x14b6, 0x6a57, 0xa56, 0x1152a, 0x1d2a, 0xd54, 0xd5aa, 0x156a, 0x96c,
                        0x94ae, 0x14ae, 0xa4c,
                        0x7d26, 0x1b2a, 0xeb55, 0xad4, 0x12da, 0xa95d, 0x95a, 0x149a, 0x9a4d, 0x1a4a, 0x11aa5,
                        0x16a8, 0x16d4,
                        0xd2da, 0x12b6, 0x936, 0x9497, 0x1496, 0x1564b, 0xd4a, 0xda8, 0xd5b4, 0x156c, 0x12ae,
                        0xa92f, 0x92e, 0xc96,
                        0x6d4a, 0x1d4a, 0x10d65, 0xb58, 0x156c, 0xb26d, 0x125c, 0x192c, 0x9a95, 0x1a94, 0x1b4a,
                        0x4b55, 0xad4,
                        0xf55b, 0x4ba, 0x125a, 0xb92b, 0x152a, 0x1694, 0x96aa, 0x15aa, 0x12ab5, 0x974, 0x14b6,
                        0xca57, 0xa56, 0x1526,
                        0x8e95, 0xd54, 0x15aa, 0x49b5, 0x96c, 0xd4ae, 0x149c, 0x1a4c, 0xbd26, 0x1aa6, 0xb54,
                        0x6d6a, 0x12da, 0x1695d,
                        0x95a, 0x149a, 0xda4b, 0x1a4a, 0x1aa4, 0xbb54, 0x16b4, 0xada, 0x495b, 0x936, 0xf497,
                        0x1496, 0x154a, 0xb6a5,
                        0xda4, 0x15b4, 0x6ab6, 0x126e, 0x1092f, 0x92e, 0xc96, 0xcd4a, 0x1d4a, 0xd64, 0x956c,
                        0x155c, 0x125c, 0x792e,
                        0x192c, 0xfa95, 0x1a94, 0x1b4a, 0xab55, 0xad4, 0x14da, 0x8a5d, 0xa5a, 0x1152b, 0x152a,
                        0x1694, 0xd6aa,
                        0x15aa, 0xab4, 0x94ba, 0x14b6, 0xa56, 0x7527, 0xd26, 0xee53, 0xd54, 0x15aa, 0xa9b5, 0x96c,
                        0x14ae, 0x8a4e,
                        0x1a4c, 0x11d26, 0x1aa4, 0x1b54, 0xcd6a, 0xada, 0x95c, 0x949d, 0x149a, 0x1a2a, 0x5b25,
                        0x1aa4, 0xfb52,
                        0x16b4, 0xaba, 0xa95b, 0x936, 0x1496, 0x9a4b, 0x154a, 0x136a5, 0xda4, 0x15ac]
    # 额外添加数据，方便快速计算阴历转阳历 每个元素的存储格式如下：
    #    12~7         6~5    4~0
    #  离元旦多少天  春节月  春节日
    #####################################################################################
    solar_1_1 = [1887, 0xec04c, 0xec23f, 0xec435, 0xec649, 0xec83e, 0xeca51, 0xecc46, 0xece3a,
                 0xed04d, 0xed242, 0xed436, 0xed64a, 0xed83f, 0xeda53, 0xedc48, 0xede3d, 0xee050, 0xee244, 0xee439,
                 0xee64d,
                 0xee842, 0xeea36, 0xeec4a, 0xeee3e, 0xef052, 0xef246, 0xef43a, 0xef64e, 0xef843, 0xefa37, 0xefc4b,
                 0xefe41,
                 0xf0054, 0xf0248, 0xf043c, 0xf0650, 0xf0845, 0xf0a38, 0xf0c4d, 0xf0e42, 0xf1037, 0xf124a, 0xf143e,
                 0xf1651,
                 0xf1846, 0xf1a3a, 0xf1c4e, 0xf1e44, 0xf2038, 0xf224b, 0xf243f, 0xf2653, 0xf2848, 0xf2a3b, 0xf2c4f,
                 0xf2e45,
                 0xf3039, 0xf324d, 0xf3442, 0xf3636, 0xf384a, 0xf3a3d, 0xf3c51, 0xf3e46, 0xf403b, 0xf424e, 0xf4443,
                 0xf4638,
                 0xf484c, 0xf4a3f, 0xf4c52, 0xf4e48, 0xf503c, 0xf524f, 0xf5445, 0xf5639, 0xf584d, 0xf5a42, 0xf5c35,
                 0xf5e49,
                 0xf603e, 0xf6251, 0xf6446, 0xf663b, 0xf684f, 0xf6a43, 0xf6c37, 0xf6e4b, 0xf703f, 0xf7252, 0xf7447,
                 0xf763c,
                 0xf7850, 0xf7a45, 0xf7c39, 0xf7e4d, 0xf8042, 0xf8254, 0xf8449, 0xf863d, 0xf8851, 0xf8a46, 0xf8c3b,
                 0xf8e4f,
                 0xf9044, 0xf9237, 0xf944a, 0xf963f, 0xf9853, 0xf9a47, 0xf9c3c, 0xf9e50, 0xfa045, 0xfa238, 0xfa44c,
                 0xfa641,
                 0xfa836, 0xfaa49, 0xfac3d, 0xfae52, 0xfb047, 0xfb23a, 0xfb44e, 0xfb643, 0xfb837, 0xfba4a, 0xfbc3f,
                 0xfbe53,
                 0xfc048, 0xfc23c, 0xfc450, 0xfc645, 0xfc839, 0xfca4c, 0xfcc41, 0xfce36, 0xfd04a, 0xfd23d, 0xfd451,
                 0xfd646,
                 0xfd83a, 0xfda4d, 0xfdc43, 0xfde37, 0xfe04b, 0xfe23f, 0xfe453, 0xfe648, 0xfe83c, 0xfea4f, 0xfec44,
                 0xfee38,
                 0xff04c, 0xff241, 0xff436, 0xff64a, 0xff83e, 0xffa51, 0xffc46, 0xffe3a, 0x10004e, 0x100242,
                 0x100437,
                 0x10064b, 0x100841, 0x100a53, 0x100c48, 0x100e3c, 0x10104f, 0x101244, 0x101438, 0x10164c,
                 0x101842, 0x101a35,
                 0x101c49, 0x101e3d, 0x102051, 0x102245, 0x10243a, 0x10264e, 0x102843, 0x102a37, 0x102c4b,
                 0x102e3f, 0x103053,
                 0x103247, 0x10343b, 0x10364f, 0x103845, 0x103a38, 0x103c4c, 0x103e42, 0x104036, 0x104249,
                 0x10443d, 0x104651,
                 0x104846, 0x104a3a, 0x104c4e, 0x104e43, 0x105038, 0x10524a, 0x10543e, 0x105652, 0x105847,
                 0x105a3b, 0x105c4f,
                 0x105e45, 0x106039, 0x10624c, 0x106441, 0x106635, 0x106849, 0x106a3d, 0x106c51, 0x106e47,
                 0x10703c, 0x10724f,
                 0x107444, 0x107638, 0x10784c, 0x107a3f, 0x107c53, 0x107e48]

    def LunarToSolar(self, lunar):
        days = LunarSolarConverter.lunar_month_days[lunar.lunarYear - LunarSolarConverter.lunar_month_days[0]]
        leap = GetBitInt(days, 4, 13)
        offset = 0
        loopend = leap
        if not lunar.isleap:

            if lunar.lunarMonth <= leap or leap == 0:

                loopend = lunar.lunarMonth - 1

            else:

                loopend = lunar.lunarMonth

        for i in range(0, loopend):
            offset += GetBitInt(days, 1, 12 - i) == 1 and 30 or 29

        offset += lunar.lunarDay

        solar11 = LunarSolarConverter.solar_1_1[lunar.lunarYear - LunarSolarConverter.solar_1_1[0]]

        y = GetBitInt(solar11, 12, 9)
        m = GetBitInt(solar11, 4, 5)
        d = GetBitInt(solar11, 5, 0)

        return SolarFromInt(SolarToInt(y, m, d) + offset - 1)

    def SolarToLunar(self, solar):

        lunar = Lunar(0, 0, 0, False)
        index = solar.solarYear - LunarSolarConverter.solar_1_1[0]
        data = (solar.solarYear << 9) | (solar.solarMonth << 5) | solar.solarDay
        if LunarSolarConverter.solar_1_1[index] > data:
            index -= 1

        solar11 = LunarSolarConverter.solar_1_1[index]
        y = GetBitInt(solar11, 12, 9)
        m = GetBitInt(solar11, 4, 5)
        d = GetBitInt(solar11, 5, 0)
        offset = SolarToInt(solar.solarYear, solar.solarMonth, solar.solarDay) - SolarToInt(y, m, d)

        days = LunarSolarConverter.lunar_month_days[index]
        leap = GetBitInt(days, 4, 13)

        lunarY = index + LunarSolarConverter.solar_1_1[0]
        lunarM = 1
        offset += 1

        for i in range(0, 13):

            dm = GetBitInt(days, 1, 12 - i) == 1 and 30 or 29
            if offset > dm:

                lunarM += 1
                offset -= dm

            else:

                break

        lunarD = int(offset)
        lunar.lunarYear = lunarY
        lunar.lunarMonth = lunarM
        lunar.isleap = False
        if leap != 0 and lunarM > leap:

            lunar.lunarMonth = lunarM - 1
            if lunarM == leap + 1:
                lunar.isleap = True

        lunar.lunarDay = lunarD
        return lunar

    def __init__(self):
        pass


if __name__ == '__main__':
    converter = LunarSolarConverter()
    solar = Solar(2111, 1, 25)
    pprint(vars(solar))
    lunar = converter.SolarToLunar(solar)
    pprint(vars(lunar))
    solar = converter.LunarToSolar(lunar)
    pprint(vars(solar))
    print(len(converter.solar_1_1))
    print("Done")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2017/11/20 16:27
# @Author  : zhm
# @File    : RangeTimeEnum.py
# @Software: PyCharm


# 范围时间的默认时间点
class RangeTimeEnum():
    day_break = 3  # 黎明
    early_morning = 8  # 早
    morning = 10  # 上午
    noon = 12  # 中午、午间
    afternoon = 15  # 下午、午后
    night = 18  # 晚上、傍晚
    lateNight = 20  # 晚、晚间
    midNight = 23  # 深夜


if __name__ == "__main__":
    print(RangeTimeEnum.afternoon)



import json
import os


class JieQi:
    """
    节气转换日期的逻辑实现
    """
    # 计算节气的C常量组
    C_list_21 = [3.87, 18.73, 5.63, 20.646, 4.81, 20.1, 5.52, 21.04, 5.678, 21.37, 7.108, 22.83, 7.5, 23.13, 7.646,
                 23.042, 8.318, 23.438, 7.438, 22.36, 7.18, 21.94, 5.4055, 20.12]

    C_list_20 = [4.6295, 19.4599, 6.3826, 21.4155, 5.59, 20.888, 6.318, 21.86, 6.5, 22.2, 7.928, 23.65, 8.35, 23.95,
                 8.44, 23.822, 9.098, 24.218, 8.218, 23.08, 7.9, 22.6, 6.11, 20.84]

    # 节气名称组
    name_Arr = ["立春", "雨水", "惊蛰", "春分", "清明", "谷雨", "立夏", "小满", "芒种", "夏至", "小暑", "大暑", "立秋", "处暑", "白露", "秋分", "寒露",
                "霜降", "立冬", "小雪", "大雪", "冬至", "小寒", "大寒"]

    def __init__(self):
        self.c_list = []

    # 特殊年份特殊节气进行纠正
    @staticmethod
    def rectify_year(year, jieqiid, day):
        # 特殊年份
        rectify_year = [2026, 2084, 1911, 2008, 1902, 1928, 1925, 2016, 1922, 2002, 1927, 1942, 2089, 2089, 1978, 1954,
                        1918, 2021, 1982, 2082, 2019, 2021]
        # 特殊节气
        rectify_jieqi = [1, 3, 6, 7, 8, 9, 10, 10, 11, 12, 14, 15, 17, 18, 19, 20, 21, 21, 22, 22, 23]
        # 偏移量
        rectify_offset = [-1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, -1, -1, 1, -1, 1]
        pop2 = -1
        if year in rectify_year:
            if year == 2089:
                pop1 = rectify_year.index(year)  # 找到位置
                pop2 = pop1 + 1
            else:
                pop1 = rectify_year.index(year)  # 找到位置

            if rectify_jieqi[pop1] == jieqiid:
                day = day + int(rectify_offset[pop1])
            if rectify_jieqi[pop2] == jieqiid:
                day = day + int(rectify_offset[pop2])
        return day

    # 计算节气日期，并创建文件
    def create_year_jieqi(self, year, key):
        year_pre = year // 100
        C_arr = 0
        if year_pre == 19:
            C_arr = self.C_list_20
        elif year_pre == 20:
            C_arr = self.C_list_21

        year_num = year % 100
        if key in self.name_Arr:
            i = self.name_Arr.index(key)
            C = C_arr[i]
            # 注意：凡闰年3月1日前闰年数要减一，即：L=[(Y-1)/4],因为小寒、大寒、立春、雨水这两个节气都小于3月1日,所以 y = y-1
            if i == 0 or i == 1 or i == 22 or i == 23:
                if self.is_leap_year(year):
                    days = (year_num * 0.2422 + C) // 1 - ((year_num - 1) // 4)
                else:
                    days = (year_num * 0.2422 + C) // 1 - (year_num // 4)
            else:
                days = (year_num * 0.2422 + C) // 1 - (year_num // 4)

            # 特殊年份节气进行纠正
            days = self.rectify_year(year, i, days)

            days = int(days)
            days = '%02d' % days
            y = int(year_num // 1)
            m = i // 2 + 2
            if m == 13:
                m = 1
            m = '%02d' % m
            y = '%02d' % y
            strs = '{3}{0}年{1}月{2}日'.format(str(y), str(m), str(days), str(year_pre))
            return strs

    # 读取年份为 name 年的节气数据
    @staticmethod
    def read_json_file(name):
        json_file = open('./json/' + name + '.json', 'r', encoding="utf-8")
        json_str = json_file.read()
        dic = json.loads(json_str)
        print(dic)

    # 读取所有年份的节气数据
    @staticmethod
    def check_all_file():
        for index in range(1900, 2100):
            c_file_name = './json/{0}.json'.format(str(index))
            if os.path.isfile(c_file_name):
                json_file = open(c_file_name, 'r', encoding="utf-8")
                json_str = json_file.read()
                dic = json.loads(json_str)
                print(str(index) + dic)

    # 算是否是闰年
    @staticmethod
    def is_leap_year(year):
        if (year % 4) != 0:
            i = 0
        elif ((year % 100) == 0) & ((year % 400) != 0):
            i = 0
        else:
            i = 1
        return i

    def jie_qi_to_date(self, year: int, jq: str) -> str:
        """
        节气转化为对应的年份
        :param year:
        :param jq:
        :return:
        """
        date = self.create_year_jieqi(year, jq)
        return date


from collections import Iterable


class linq(object):
    def __init__(self, iterable):
        if isinstance(iterable, list):
            iterable = iter(iterable)
        if not isinstance(iterable, Iterable):
            raise TypeError('参数类型错误')
        self.__end = False
        self.__iterable = iterable

    def __get_iterable(self):
        if self.__end:
            raise StopIteration('迭代器不可重复使用')

        self.__end = True
        return self.__iterable

    def all(self, predicate):
        return all(map(predicate, self.__get_iterable()))

    def any(self, predicate=None):
        return any(map(predicate, self.__get_iterable()))

    def where(self, predicate):
        return linq(filter(predicate, self.__get_iterable()))

    def max(self):
        return max(self.__get_iterable())

    def min(self):
        return min(self.__get_iterable())

    def average(self):
        count = 0
        total = 0.0
        for number in self.__get_iterable():
            total += number
            count += 1
        return total / count

    def union(self, iterable):
        return linq(self.__union(iterable))

    def __union(self, iterable):
        for x in self.__get_iterable():
            yield x
        for x in iterable:
            yield x

    def count(self):
        count = 0
        for number in self.__get_iterable():
            count += 1
        return count

    def first(self, default=None):
        return next(self.__get_iterable(), default)

    def last(self, default=None):
        it = self.__get_iterable()
        x = None
        b = False
        while True:
            try:
                x = next(it)
                b = True
            except StopIteration:
                if b:
                    return x
                return default

    def sum(self):
        return sum(self.__get_iterable())

    def tolist(self):
        return list(self.__get_iterable())

    def select(self, selector):
        return linq(map(selector, self.__get_iterable()))

    def order_by(self, predicate=None, default=False):
        return linq(iter(sorted(self.__get_iterable(), reverse=default, key=predicate)))

    def distinct(self):
        news_li = []
        for i in self.__get_iterable():
            if i not in news_li:
                news_li.append(i)
        return linq(news_li)

    def group_by(self, predicate=None):
        data = dict()
        s = list()
        if predicate:
            for dic in self.__get_iterable():
                if dic.get(predicate) in s:
                    data[dic.get(predicate)].append(dic)
                else:
                    l = list()
                    l.append(dic)
                    data[dic.get(predicate)] = l
                    s.append(dic.get(predicate))
            return data
        else:
            for dic in self.__get_iterable():
                for k, v in dic.items():
                    if k in s:
                        data[k].append(v)
                    else:
                        l = list()
                        l.append(v)
                        data[k] = l
                        s.append(k)
            return data

    def limit(self, count):
        return linq(iter(list(self.__get_iterable())[:count]))



import math


class TimeCodeConvertHelper(object):
    """
    时码转换帮助类
    """
    # endregion

    # region 25

    m_mpcStRate25 = 50  # PAL field frequency

    MpcStFrameRate25 = 25  # PAL frame  frequency

    MpcStScale25 = 1  # PAL scale

    # endregion

    # region 2997

    m_mpcStRate2997 = 60000  # NTSC field frequency

    MpcStFrameRate2997 = 30000  # NTSC frame  frequency

    MpcStScale2997 = 1001  # NTSC scale

    # endregion

    # region 30

    m_mpcStRate30 = 60  # 30-F field frequency

    MpcStFrameRate30 = 30  # 30-F frame frequency

    MpcStScale30 = 1  # 30-F scale

    # endregion

    # region 24

    m_mpcStRate24 = 48  # 24-F field frequency

    MpcStFrameRate24 = 24  # 24-F field frequency

    MpcStScale24 = 1  # 24-F scale

    # endregion

    # region 2398

    m_mpcStRate2398 = 48000  # 2398-F field frequency

    MpcStFrameRate2398 = 24000  # 2398-F frame frequency

    MpcStScale2398 = 1001  # 2398-F scale

    # endregion

    # region 50

    m_mpcStRate50 = 50  # PAL field frequency

    MpcStFrameRate50 = 50  # PAL frame  frequency

    MpcStScale50 = 1  # PAL scale

    # endregion

    # region 5994

    m_mpcStRate5994 = 60000  # NTSC field frequency

    MpcStFrameRate5994 = 60000  # NTSC frame  frequency

    MpcStScale5994 = 1001  # NTSC scale

    # endregion

    # region 60

    m_mpcStRate60 = 60  # 60-F field frequency

    MpcStFrameRate60 = 60  # 60-F frame frequency

    MpcStScale60 = 1  # 60-F scale

    # endregion

    # region 25 Frame

    MpcFramesSecond25 = 25  # 25 Frame: frames per second

    MpcFramesMinute25 = 1500  # 25 Frame: frames per minute

    MpcFramesHour25 = 90000  # 25 Frame: frames per hour

    # endregion

    # region 24 DROP Frame

    MpcFramesMinute24Drop = 1438  # 24 DROP Frame: frames per minute

    MpcFrames10Minutes24Drop = 14382  # 24 DROP Frame: frames per 10 minutes

    MpcFramesHour24Drop = 86292  # 24 DROP Frame: frames per hour

    MpcFramesSecond24 = 24  # 24 Frame: frames per second

    MpcFramesMinute24 = 1440  # 24 Frame: frames per minute

    MpcFramesHour24 = 86400  # 24 Frame: frames per hour

    MpcFramesSecondNodrop30 = 30  # 30 NO_DROP Frame: frames per second

    MpcFramesMinuteNodrop30 = 1800  # 30 NO_DROP Frame: frames per minute

    MpcFramesHourNodrop30 = 108000  # 30 NO_DROP Frame: frames per hour

    # endregion

    # region 30 DROP Frame

    MpcFramesMinute30Drop = 1798  # 30 DROP Frame: frames per minute

    MpcFrames10Minutes30Drop = 17982  # 30 DROP Frame: frames per 10 minutes

    MpcFramesHour30Drop = 107892  # 30 DROP Frame: frames per hour

    # endregion

    # region 50 Frame

    MpcFramesSecond50 = 50  # 25 Frame: frames per second

    MpcFramesMinute50 = 3000  # 25 Frame: frames per minute

    MpcFramesHour50 = 180000  # 25 Frame: frames per hour

    # endregion

    # region 60 DROP Frame

    MpcFramesMinute60Drop = 3596  # 60 DROP Frame: frames per minute

    MpcFrames10Minutes60Drop = 35964  # 60 DROP Frame: frames per 10 minutes

    MpcFramesHour60Drop = 215784  # 60 DROP Frame: frames per hour

    # endregion

    # region 60 Frame

    MpcFramesSecond60 = 60  # 60 Frame: frames per second

    MpcFramesMinute60 = 3600  # 60 Frame: frames per minute

    MpcFramesHour60 = 216000  # 60 Frame: frames per hour

    @staticmethod
    def __data_type_checker(args):
        """
        数据类型检查器
        :param args:
        :return:
        """
        for k, v in args:
            if not isinstance(k, v):
                if v == float and isinstance(k, int):
                    pass
                else:
                    raise TypeError('传入的参数类型错误')

    def Frame2L100NsByScale(self, lFrame, dRate, dScale):
        """
        帧转百纳秒
        :param lFrame: 帧
        :param dRate: 帧率
        :param dScale:
        :return:百纳秒
        """
        self.__data_type_checker(((lFrame, int), (dRate, float), (dScale, float)))
        dFrameRate = float(self.MpcStFrameRate25)
        dFrameScale = float(self.MpcStScale25)
        dFrameRate, dFrameScale = self.__Rate2ScaleFrameRateAndFrameScale(dRate, dScale, dFrameRate, dFrameScale)
        return int(math.floor(lFrame * pow(10, 7) * dFrameRate / dFrameScale))

    def Frame2L100Ns(self, dFrame, dFrameRate):
        """
        帧转百纳秒
        :param dFrame: 帧
        :param dFrameRate: 帧率
        :return:
        """
        self.__data_type_checker(((dFrame, int), (dFrameRate, float)))
        dRate = float(self.MpcStFrameRate25)
        dScale = float(self.MpcStScale25)
        dRate, dScale = self.__FrameRate2RateAndScale(dFrameRate, dRate, dScale)
        return int(math.floor(dFrame * dScale * pow(10, 7) / dRate))

    def __Rate2ScaleFrameRateAndFrameScale(self, dRate, dScale, dFrameRate, dFrameScale):
        """
        帧率修正
        :param dRate: 帧率
        :param dScale: 修正值
        :param dFrameRate:
        :param dFrameScale:
        :return:
        """
        self.__data_type_checker(((dRate, float), (dScale, float), (dFrameRate, float), (dFrameScale, float)))
        if ((dRate == self.MpcStFrameRate25 and dScale == self.MpcStScale25) or (
                dRate * self.MpcStScale25 == dScale * self.MpcStFrameRate25)):
            dFrameRate = self.MpcStFrameRate25
            dFrameScale = self.MpcStScale25
        elif ((dRate == self.MpcStFrameRate2997 and dScale == self.MpcStScale2997) or (
                dRate * self.MpcStScale2997 == dScale * self.MpcStFrameRate2997)):
            dFrameRate = self.MpcStFrameRate2997
            dFrameScale = self.MpcStScale2997
        elif ((dRate == self.MpcStFrameRate30 and dScale == self.MpcStScale30) or (
                dRate * self.MpcStScale30 == dScale * self.MpcStFrameRate30)):
            dFrameRate = self.MpcStFrameRate30
            dFrameScale = self.MpcStScale30
        elif ((dRate == self.MpcStFrameRate24 and dScale == self.MpcStScale24) or (
                dRate * self.MpcStScale24 == dScale * self.MpcStFrameRate24)):
            dFrameRate = self.MpcStFrameRate24
            dFrameScale = self.MpcStScale24
        elif ((dRate == self.MpcStFrameRate2398 and dScale == self.MpcStScale2398) or (
                dRate * self.MpcStScale2398 == dScale * self.MpcStFrameRate2398)):
            dFrameRate = self.MpcStFrameRate2398
            dFrameScale = self.MpcStScale2398
        elif ((dRate == self.MpcStFrameRate50 and dScale == self.MpcStScale50) or (
                dRate * self.MpcStScale50 == dScale * self.MpcStFrameRate50)):
            dFrameRate = self.MpcStFrameRate50
            dFrameScale = self.MpcStScale50
        elif ((dRate == self.MpcStFrameRate5994 and dScale == self.MpcStScale5994) or (
                dRate * self.MpcStScale5994 == dScale * self.MpcStFrameRate5994)):
            dFrameRate = self.MpcStFrameRate5994
            dFrameScale = self.MpcStScale5994
        elif ((dRate == self.MpcStFrameRate60 and dScale == self.MpcStScale60) or (
                dRate * self.MpcStScale60 == dScale * self.MpcStFrameRate60)):
            dFrameRate = self.MpcStFrameRate60
            dFrameScale = self.MpcStScale60
        return dFrameRate, dFrameScale

    def __FrameRate2RateAndScale(self, dFrameRate, dRate, dScale):
        """
        获取帧率和修正值
        :param dFrameRate:输入帧率
        :param dRate:
        :param dScale:
        :return:
        """
        self.__data_type_checker(((dFrameRate, float), (dRate, float), (dScale, float)))
        if (abs(dFrameRate - self.MpcStFrameRate25 / float
            (self.MpcStScale25)) < 0.01):
            dRate = self.MpcStFrameRate25
            dScale = self.MpcStScale25
        elif (abs(dFrameRate - self.MpcStFrameRate2997 / float
            (self.MpcStScale2997)) < 0.01):
            dRate = self.MpcStFrameRate2997
            dScale = self.MpcStScale2997
        elif (abs(dFrameRate - self.MpcStFrameRate30 / float
            (self.MpcStScale30)) < 0.01):
            dRate = self.MpcStFrameRate30
            dScale = self.MpcStScale30
        elif (abs(dFrameRate - self.MpcStFrameRate24 / float
            (self.MpcStScale24)) < 0.01):
            dRate = self.MpcStFrameRate24
            dScale = self.MpcStScale24
        elif (abs(dFrameRate - self.MpcStFrameRate2398 / float
            (self.MpcStScale2398)) < 0.01):
            dRate = self.MpcStFrameRate2398
            dScale = self.MpcStScale2398
        elif (abs(dFrameRate - self.MpcStFrameRate50 / float
            (self.MpcStScale50)) < 0.01):
            dRate = self.MpcStFrameRate50
            dScale = self.MpcStScale50
        elif (abs(dFrameRate - self.MpcStFrameRate5994 / float
            (self.MpcStScale5994)) < 0.01):
            dRate = self.MpcStFrameRate5994
            dScale = self.MpcStScale5994
        elif (abs(dFrameRate - self.MpcStFrameRate60 / float
            (self.MpcStScale60)) < 0.01):
            dRate = self.MpcStFrameRate60
            dScale = self.MpcStScale60
        return dRate, dScale

    def L100Ns2FrameByScale(self, l100Ns, dRate, dScale):
        """
        百纳秒转帧
        :param l100Ns: 百纳秒
        :param dRate: 帧率
        :param dScale: 修正值
        :return:
        """
        self.__data_type_checker(((l100Ns, int), (dRate, float), (dScale, float)))
        dFrameRate = float(self.MpcStFrameRate25)
        dFrameScale = float(self.MpcStScale25)
        dFrameRate, dFrameScale = self.__Rate2ScaleFrameRateAndFrameScale(dRate, dScale, dFrameRate, dFrameScale)
        return int(math.floor(l100Ns / math.pow(10, 7) * dFrameRate / dFrameScale + 0.5))

    def L100Ns2Frame(self, l100Ns, dFrameRate):
        """
        百纳秒转帧
        :param l100Ns: 百纳秒
        :param dFrameRate: 帧率
        :return:
        """
        self.__data_type_checker(((l100Ns, int), (dFrameRate, float)))
        dRate = float(self.MpcStFrameRate25)
        dScale = float(self.MpcStScale25)
        dRate, dScale = self.__FrameRate2RateAndScale(dFrameRate, dRate, dScale)
        return int(math.floor(l100Ns * dRate / dScale / math.pow(10, 7) + 0.5))

    def Frame2TcByScale(self, dFrame, dRate, dScale, dropFrame):
        """
        帧转时码
        :param dFrame: 帧
        :param dRate: 帧率
        :param dScale:
        :param dropFrame:
        :return: 时码字符串
        """
        self.__data_type_checker(((dFrame, float), (dRate, float), (dScale, float), (dropFrame, bool)))
        strTc = ''
        if ((dRate == self.MpcStFrameRate25 and dScale == self.MpcStScale25) or (
                dRate * self.MpcStScale25 == dScale * self.MpcStFrameRate25)):
            dHour = int((math.floor(dFrame / self.MpcFramesHour25)))
            dResidue = int((math.floor(dFrame % self.MpcFramesHour25)))
            dMin = int((math.floor(float(dResidue) / self.MpcFramesMinute25)))
            dResidue = dResidue % self.MpcFramesMinute25
            dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond25))
            dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond25))
            strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        elif ((dRate == self.MpcStFrameRate2997 and dScale == self.MpcStScale2997) or (
                dRate * self.MpcStScale2997 == dScale * self.MpcStFrameRate2997)):
            if dropFrame:
                dHour = int(math.floor(dFrame / self.MpcFramesHour30Drop))
                dResidue = int(math.floor(dFrame % self.MpcFramesHour30Drop))
                dMin = int(math.floor(float(10) * (dResidue / self.MpcFrames10Minutes30Drop)))
                dResidue = dResidue % self.MpcFrames10Minutes30Drop
                if dResidue >= self.MpcFramesMinuteNodrop30:
                    dResidue -= self.MpcFramesMinuteNodrop30
                    dMin += 1 + dResidue / self.MpcFramesMinute30Drop
                    dResidue %= self.MpcFramesMinute30Drop
                    dResidue += 2
                dSec = int(math.floor(float(dResidue) / self.MpcFramesSecondNodrop30))
                dFra = int(math.floor(float(dResidue) % self.MpcFramesSecondNodrop30))
                strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, True)
            else:
                dHour = int(math.floor(float(dFrame) / self.MpcFramesHourNodrop30))
                dResidue = int(math.floor(float(dFrame) % self.MpcFramesHourNodrop30))
                dMin = int(math.floor(float(dResidue) / self.MpcFramesMinuteNodrop30))
                dResidue = dResidue % self.MpcFramesMinuteNodrop30
                dSec = int(math.floor(float(dResidue) / self.MpcFramesSecondNodrop30))
                dFra = int(math.floor(float(dResidue) % self.MpcFramesSecondNodrop30))
                strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        elif ((dRate == self.MpcStFrameRate30 and dScale == self.MpcStScale30) or (
                dRate * self.MpcStScale30 == dScale * self.MpcStFrameRate30)):
            dHour = int(math.floor(float(dFrame) / self.MpcFramesHourNodrop30))
            dResidue = int(math.floor(float(dFrame) % self.MpcFramesHourNodrop30))
            dMin = int(math.floor(float(dResidue) / self.MpcFramesMinuteNodrop30))
            dResidue = dResidue % self.MpcFramesMinuteNodrop30
            dSec = int(math.floor(float(dResidue) / self.MpcFramesSecondNodrop30))
            dFra = int(math.floor(float(dResidue) % self.MpcFramesSecondNodrop30))
            strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        elif ((dRate == self.MpcStFrameRate24 and dScale == self.MpcStScale24) or (
                dRate * self.MpcStScale24 == dScale * self.MpcStFrameRate24)):
            dHour = int(math.floor(float(dFrame) / self.MpcFramesHour24))
            dResidue = int(math.floor(float(dFrame) % self.MpcFramesHour24))
            dMin = int(math.floor(float(dResidue) / self.MpcFramesMinute24))
            dResidue = dResidue % self.MpcFramesMinute24
            dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond24))
            dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond24))
            strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        elif ((dRate == self.MpcStFrameRate2398 and dScale == self.MpcStScale2398) or (
                dRate * self.MpcStScale2398 == dScale * self.MpcStFrameRate2398)):
            if dropFrame:
                dHour = int(math.floor(dFrame / self.MpcFramesHour24Drop))
                dResidue = int(math.floor(dFrame % self.MpcFramesHour24Drop))
                dMin = int(math.floor(float(10) * (dResidue / self.MpcFrames10Minutes24Drop)))
                dResidue = dResidue % self.MpcFrames10Minutes24Drop
                if dResidue >= self.MpcFramesMinute24:
                    dResidue -= self.MpcFramesMinute24
                    dMin += 1 + dResidue / self.MpcFramesMinute24Drop
                    dResidue %= self.MpcFramesMinute24Drop
                    dResidue += 2
                dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond24))
                dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond24))
                strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, True)
            else:
                dHour = int(math.floor(float(dFrame) / self.MpcFramesHour24))
                dResidue = int(math.floor(float(dFrame) % self.MpcFramesHour24))
                dMin = int(math.floor(float(dResidue) / self.MpcFramesMinute24))
                dResidue = dResidue % self.MpcFramesMinute24
                dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond24))
                dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond24))
                strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        elif ((dRate == self.MpcStFrameRate50 and dScale == self.MpcStScale50) or (
                dRate * self.MpcStScale50 == dScale * self.MpcStFrameRate50)):
            dHour = int(math.floor(dFrame / self.MpcFramesHour50))
            dResidue = int(math.floor(dFrame % self.MpcFramesHour50))
            dMin = int(math.floor(float(dResidue) / self.MpcFramesMinute50))
            dResidue = dResidue % self.MpcFramesMinute50
            dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond50))
            dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond50))
            strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        elif ((dRate == self.MpcStFrameRate5994 and dScale == self.MpcStScale5994) or (
                dRate * self.MpcStScale5994 == dScale * self.MpcStFrameRate5994)):
            if dropFrame:
                dHour = int(math.floor(dFrame / self.MpcFramesHour60Drop))
                dResidue = int(math.floor(dFrame % self.MpcFramesHour60Drop))
                dMin = int(math.floor(float(10) * (dResidue / self.MpcFrames10Minutes60Drop)))
                dResidue = dResidue % self.MpcFrames10Minutes60Drop
                if dResidue >= self.MpcFramesMinute60:
                    dResidue -= self.MpcFramesMinute60
                    dMin += 1 + dResidue / self.MpcFramesMinute60Drop
                    dResidue %= self.MpcFramesMinute60Drop
                    dResidue += 4
                dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond60))
                dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond60))
                strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, True)
            else:
                dHour = int(math.floor(float(dFrame) / self.MpcFramesHour60))
                dResidue = int(math.floor(float(dFrame) % self.MpcFramesHour60))
                dMin = int(math.floor(float(dResidue) / self.MpcFramesMinute60))
                dResidue = dResidue % self.MpcFramesMinute60
                dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond60))
                dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond60))
                strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)

        elif ((dRate == self.MpcStFrameRate60 and dScale == self.MpcStScale60) or (
                dRate * self.MpcStScale60 == dScale * self.MpcStFrameRate60)):
            dHour = int(math.floor(float(dFrame) / self.MpcFramesHour60))
            dResidue = int(math.floor(float(dFrame) % self.MpcFramesHour60))
            dMin = int(math.floor(float(dResidue) / self.MpcFramesMinute60))
            dResidue = dResidue % self.MpcFramesMinute60
            dSec = int(math.floor(float(dResidue) / self.MpcFramesSecond60))
            dFra = int(math.floor(float(dResidue) % self.MpcFramesSecond60))
            strTc = self.__FormatTimeCodeString(dHour, dMin, dSec, dFra, False)
        return strTc

    def Frame2Tc(self, dFrame, dFrameRate, dropFrame):
        """
        帧转时码
        :param dFrame: 帧
        :param dFrameRate:帧率
        :param dropFrame: 是否丢帧
        :return:
        """
        self.__data_type_checker(((dFrame, int), (dFrameRate, float), (dropFrame, bool)))
        dRate = self.MpcStFrameRate25
        dScale = self.MpcStScale25
        dRate, dScale = self.__FrameRate2RateAndScale(dFrameRate, dRate, dScale)
        strTc = self.Frame2TcByScale(dFrame, dRate, dScale, dropFrame)
        return strTc

    def __FormatTimeCodeString(self, hours, minutes, seconds, frames, dropFrame):
        """
        格式化时码字符串
        :param hours: 小时数
        :param minutes: 分指数
        :param seconds: 秒数
        :param frames: 帧数
        :param dropFrame: 是否是丢帧
        :return: 格式化后的时码字符串
        """
        self.__data_type_checker(((hours, int), (minutes, int), (seconds, int), (frames, int), (dropFrame, bool)))
        hours = hours - 24 if hours >= 24 else hours
        minutes = minutes - 60 if minutes >= 60 else minutes
        seconds = seconds - 60 if seconds >= 60 else seconds
        framesSeparator = "."
        if not dropFrame:
            framesSeparator = ":"
        hours = int(math.floor(hours % 24.0))
        return "{0:0>2d}:{1:0>2d}{4}{2:0>2d}:{3:0>2d}".format(hours, minutes, seconds, frames, framesSeparator)

    def TimeCode2Frame(self, sTimeCode, dFrameRate, dropFrame):
        """
        时间字符串转帧
        :param sTimeCode: 时码
        :param dFrameRate: 帧率
        :param dropFrame: 是否是丢帧
        :return:
        """
        self.__data_type_checker(((sTimeCode, str), (dFrameRate, float), (dropFrame, bool)))
        sTimeCode = self.__FormatTimeCodeStringByFrame(sTimeCode, dFrameRate, dropFrame)
        dRate = float(self.MpcStFrameRate25)
        dScale = float(self.MpcStScale25)
        dRate, dScale = self.__FrameRate2RateAndScale(dFrameRate, dRate, dScale)
        ftcFrames = self.TimeCode2FrameByScale(sTimeCode, dFrameRate, dRate, dScale, dropFrame)
        newTc = self.Frame2Tc(ftcFrames, dFrameRate, dropFrame)
        newTc = newTc.replace(".", ":")
        sTimeCode = sTimeCode.replace(".", ":")
        if newTc != sTimeCode:
            ftcFrames = self.__GetFrameByTimeCode(sTimeCode, ftcFrames, True, 1, dFrameRate, dropFrame)
        return ftcFrames

    def __FormatTimeCodeStringByFrame(self, timeCode, dFrameRate, dropFrame):
        """
        格式化时码字符串
        :param timeCode: 时间字符串
        :param dFrameRate: 帧率
        :param dropFrame: 是否是丢帧
        :return:
        """
        self.__data_type_checker(((timeCode, str), (dFrameRate, float), (dropFrame, bool)))
        if timeCode:
            if len(timeCode) == 11:
                c = list(timeCode)
                if c[8] == '.':
                    c[8] = ':'
                    c[5] = '.'
                timeCode = ''.join(c)
            hours = 0
            minutes = 0
            seconds = 0
            frames = 0
            ftcs = timeCode.split(':')
            hours = int(ftcs[0])
            if len(ftcs) >= 4:
                minutes = int(ftcs[1])
                seconds = int(ftcs[2])
                if int(ftcs[3]) >= dFrameRate:
                    showframeRate = float(math.ceil(dFrameRate) - 1)
                    ftcs[3] = str(showframeRate)
                else:
                    if dropFrame:
                        dropM = ["00", "10", "20", "30", "40", "50"]
                        drop5994F = ["00", "01", "02", "03"]
                        dropF = ["00", "01"]
                        if ftcs[2] == "00" and ftcs[1] not in dropM and ftcs[3] in drop5994F:
                            if abs(60.0 - dFrameRate) < 0.1:
                                ftcs[3] = "04"
                            elif abs(30.0 - dFrameRate) < 0.1 or abs(24.0 - dFrameRate) < 0.1:
                                if ftcs[3] in dropF:
                                    ftcs[3] = "02"
                frames = int(ftcs[3])
            else:
                ftcssf = ftcs[1].split('.')
                minutes = int(ftcssf[0])
                seconds = int(ftcssf[1])
                if int(ftcs[2]) >= dFrameRate:
                    # region 修正最后一位 29.97最大显示29 25最大显示24
                    showframeRate = float(math.ceil(dFrameRate) - 1)
                    ftcs[2] = str(showframeRate)
                else:
                    # region 验证是否是丢帧时码
                    if dropFrame:
                        dropM = ["00", "10", "20", "30", "40", "50"]
                        drop5994F = ["00", "01", "02", "03"]
                        dropF = ["00", "01"]
                        if ftcssf[1] == "00" and dropM not in ftcssf[0] and ftcs[2] in drop5994F:
                            if abs(60.0 - dFrameRate) < 0.1:
                                ftcs[2] = "04"
                            elif abs(30.0 - dFrameRate) < 0.1 or abs(24.0 - dFrameRate) < 0.1:
                                if ftcs[2] in dropF:
                                    ftcs[2] = "02"
                frames = int(ftcs[2])
            return self.__FormatTimeCodeString(hours, minutes, seconds, frames, dropFrame)
        return "--:--:--:--"

    def TimeCode2FrameByScale(self, sTimeCode, frameRate, dRate, dScale, dropframe):
        """
        时间字符串转帧
        :param sTimeCode:时码
        :param frameRate:帧率
        :param dRate:
        :param dScale:
        :param dropframe:是否是丢帧
        :return:帧
        """
        self.__data_type_checker(
            ((sTimeCode, str), (frameRate, float), (dRate, float), (dScale, float), (dropframe, bool)))
        ftcFrames = 0
        if ((dRate == self.MpcStFrameRate25 and dScale == self.MpcStScale25) or (
                dRate * self.MpcStScale25 == dScale * self.MpcStFrameRate25)):
            ftcFrames = self.TimeCode2NdfFrame(sTimeCode, frameRate)
        elif ((dRate == self.MpcStFrameRate2997 and dScale == self.MpcStScale2997) or (
                dRate * self.MpcStScale2997 == dScale * self.MpcStFrameRate2997)):
            lHour = 0
            lMinute = 0
            lSecond = 0
            lFrame = 0
            lHour, lMinute, lSecond, lFrame = self.__TimeCode2Format(sTimeCode, lHour, lMinute, lSecond, lFrame,
                                                                     frameRate)
            if dropframe:
                ftcFrames += lHour * self.MpcFramesHour30Drop
                lwReste = int(lMinute / 10)
                ftcFrames += lwReste * self.MpcFrames10Minutes30Drop
                lwReste = lMinute % 10
                if lwReste > 0:
                    ftcFrames += self.MpcFramesMinuteNodrop30
                    ftcFrames += (lwReste - 1) * self.MpcFramesMinute30Drop
                    ftcFrames -= 2
                ftcFrames += lSecond * self.MpcFramesSecondNodrop30
                ftcFrames += lFrame
            else:
                ftcFrames = self.TimeCode2NdfFrame(sTimeCode, 30)
        elif ((dRate == self.MpcStFrameRate30 and dScale == self.MpcStScale30) or (
                dRate * self.MpcStScale30 == dScale * self.MpcStFrameRate30)):
            ftcFrames = self.TimeCode2NdfFrame(sTimeCode, frameRate)
        elif ((dRate == self.MpcStFrameRate24 and dScale == self.MpcStScale24) or (
                dRate * self.MpcStScale24 == dScale * self.MpcStFrameRate24)):
            ftcFrames = self.TimeCode2NdfFrame(sTimeCode, frameRate)
        elif ((dRate == self.MpcStFrameRate2398 and dScale == self.MpcStScale2398) or (
                dRate * self.MpcStScale2398 == dScale * self.MpcStFrameRate2398)):
            lHour = 0
            lMinute = 0
            lSecond = 0
            lFrame = 0
            lHour, lMinute, lSecond, lFrame = self.__TimeCode2Format(sTimeCode, lHour, lMinute, lSecond, lFrame,
                                                                     frameRate)
            if dropframe:
                ftcFrames += lHour * self.MpcFramesHour24
                lwReste = int(lMinute / 10)
                ftcFrames += lwReste * self.MpcFrames10Minutes24Drop
                lwReste = lMinute % 10
                if lwReste > 0:
                    ftcFrames += self.MpcFramesMinute60
                    ftcFrames += (lwReste - 1) * self.MpcFramesMinute24
                    ftcFrames -= 2
                ftcFrames += lSecond * self.MpcFramesSecond24
                ftcFrames += lFrame
            else:
                ftcFrames = self.TimeCode2NdfFrame(sTimeCode, 24)
        elif ((dRate == self.MpcStFrameRate50 and dScale == self.MpcStScale50) or (
                dRate * self.MpcStScale50 == dScale * self.MpcStFrameRate50)):
            ftcFrames = self.TimeCode2NdfFrame(sTimeCode, frameRate)
        elif ((dRate == self.MpcStFrameRate5994 and dScale == self.MpcStScale5994) or (
                dRate * self.MpcStScale5994 == dScale * self.MpcStFrameRate5994)):
            lHour = 0
            lMinute = 0
            lSecond = 0
            lFrame = 0
            lHour, lMinute, lSecond, lFrame = self.__TimeCode2Format(sTimeCode, lHour, lMinute, lSecond, lFrame,
                                                                     frameRate)
            if dropframe:
                ftcFrames += lHour * self.MpcFramesHour60Drop
                lwReste = int(lMinute / 10)
                ftcFrames += lwReste * self.MpcFrames10Minutes60Drop
                lwReste = lMinute % 10
                if lwReste > 0:
                    ftcFrames += self.MpcFramesMinute60
                    ftcFrames += (lwReste - 1) * self.MpcFramesMinute60Drop
                    ftcFrames -= 4
                ftcFrames += lSecond * self.MpcFramesSecond60
                ftcFrames += lFrame
            else:
                ftcFrames = self.TimeCode2NdfFrame(sTimeCode, 60)
        elif ((dRate == self.MpcStFrameRate60 and dScale == self.MpcStScale60) or (
                dRate * self.MpcStScale60 == dScale * self.MpcStFrameRate60)):
            ftcFrames = self.TimeCode2NdfFrame(sTimeCode, frameRate)
        return ftcFrames

    def __GetFrameByTimeCode(self, sTimeCode, ftcFrames, isAdded, corrValue, dFrameRate, dropFrame):
        """
        递归解决时码丢帧的问题
        :param sTimeCode:
        :param ftcFrames:
        :param isAdded:是否加修正值
        :param corrValue:修正值
        :param dFrameRate:
        :param dropFrame:
        :return:
        """
        self.__data_type_checker(
            ((sTimeCode, str), (ftcFrames, int), (isAdded, bool), (corrValue, int), (dFrameRate, float),
             (dropFrame, bool)))
        ftcNewFrames = 0
        if isAdded:
            ftcNewFrames = ftcFrames + corrValue
        else:
            ftcNewFrames = ftcFrames - corrValue
            corrValue += 1
        newTc = self.Frame2Tc(ftcNewFrames, dFrameRate, dropFrame)
        newTc = newTc.replace(".", ":")
        sTimeCode = sTimeCode.replace(".", ":")
        if newTc != sTimeCode:
            return self.__GetFrameByTimeCode(sTimeCode, ftcFrames, not isAdded, corrValue, dFrameRate, dropFrame)
        return ftcNewFrames

    def __TimeCode2Format(self, sTimeCode, lHour, lMinute, lSecond, lFrame, dFrameRate):
        """
        时间字符串格式化
        :param sTimeCode:
        :param lHour:
        :param lMinute:
        :param lSecond:
        :param lFrame:
        :param dFrameRate:
        :return: 帧数
        """
        self.__data_type_checker(
            ((sTimeCode, str), (lHour, int), (lMinute, int), (lSecond, int), (lFrame, int), (dFrameRate, float)))
        ftcCodes = sTimeCode.split(':')
        if len(ftcCodes) >= 4:
            lHour = int(ftcCodes[0])
            lMinute = int(ftcCodes[1])
            lSecond = int(ftcCodes[2])
            lFrame = int(ftcCodes[3])
        else:
            ftcssf = ftcCodes[1].split('.')
            lHour = int(ftcCodes[0])
            lMinute = int(ftcssf[0])
            lSecond = int(ftcssf[1])
            lFrame = int(ftcCodes[2])
        lHour = lHour - 24 if lHour >= 24 else lHour
        lMinute = lMinute - 60 if lMinute >= 60 else lMinute
        lSecond = lSecond - 60 if lSecond >= 60 else lSecond
        lFrame = lFrame - int(math.ceil(dFrameRate)) if lFrame >= math.ceil(dFrameRate) else lFrame
        return lHour, lMinute, lSecond, lFrame

    def TimeCode2NdfFrame(self, sTimeCode, dFrameRate):
        """
        时间字符串转秒(未考虑丢帧的情况)
        :param sTimeCode:
        :param dFrameRate:
        :return:帧数
        """
        self.__data_type_checker(((sTimeCode, str), (dFrameRate, float)))
        ftcSeconds = 0
        ftcFrames = 0
        lHour = 0
        lMinute = 0
        lSecond = 0
        lFrame = 0
        lHour, lMinute, lSecond, lFrame = self.__TimeCode2Format(sTimeCode, lHour, lMinute, lSecond, lFrame, dFrameRate)
        ftcSeconds += lHour * 60 * 60
        ftcSeconds += lMinute * 60
        ftcSeconds += lSecond
        ftcFrames += lFrame
        ftcFrames += self.Second2Frame(ftcSeconds, dFrameRate)
        return ftcFrames

    def Second2Frame(self, dbSec, dFrameRate):
        """
        秒转帧
        :param dbSec:秒数
        :param dFrameRate:帧率
        :return:帧数
        """
        self.__data_type_checker(((dbSec, float), (dFrameRate, float)))
        if dbSec >= 24 * 60 * 60:
            dbSec = dbSec - 24 * 60 * 60
            return self.Second2Frame(dbSec, dFrameRate)
        dbSec = math.ceil(dbSec * math.pow(10, 7))
        dRate = float(self.MpcStFrameRate25)
        dScale = float(self.MpcStScale25)
        dRate, dScale = self.__FrameRate2RateAndScale(dFrameRate, dRate, dScale)
        return int(math.floor(dbSec * dRate / dScale / math.pow(10, 7)))

    def L100Ns2Tc(self, l100Ns, dropFrame):
        """
        百纳秒转时码
        :param l100Ns: 百纳秒
        :param dropFrame: 是否是丢帧
        :return: 时码字符串
        """
        self.__data_type_checker(((l100Ns, int), (dropFrame, bool)))
        dHour = int(math.floor(l100Ns / (60 * 60 * math.pow(10, 7))))
        llResidue = int(l100Ns % (60 * 60 * math.pow(10, 7)))
        dMin = int(math.floor(llResidue / (60 * math.pow(10, 7))))
        llResidue = llResidue % int(math.floor(60 * math.pow(10, 7)))
        dSec = int(math.floor(llResidue / (math.pow(10, 7))))
        llResidue = llResidue % int(math.pow(10, 7))
        dFraction = int(math.floor(float(llResidue) / (10 * 1000 * 10)))
        return self.__FormatTimeCodeString(dHour, dMin, dSec, dFraction, dropFrame)

    def L100Ns2TcByFrame(self, l100Ns, dFrameRate, dropFrame):
        """
        百纳秒转时码
        :param l100Ns: 百纳秒
        :param dFrameRate: 帧率
        :param dropFrame:
        :return: 时码字符串
        """
        self.__data_type_checker(((l100Ns, int), (dFrameRate, float), (dropFrame, bool)))
        return self.Frame2Tc(self.L100Ns2Frame(l100Ns, dFrameRate), dFrameRate, dropFrame)


# -*- coding: utf-8 -*-

# 初始化
global _global_dict
_global_dict = {}


def set_value(key, value):
    """ 定义一个全局变量 """
    _global_dict[key] = value


def get_value(key, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return _global_dict[key]
    except KeyError:
        return defValue



#!/usr/bin/python3
# -*- coding:utf-8 -*-

from collections import defaultdict


class MyDefaultDict(defaultdict):
    """
    利用defaultdict实现树结构，考虑树节点的计数问题
    """

    def __init__(self, default_factory=None, num=0):
        super().__init__(default_factory)
        self.num = num


class MyTreeClass:
    """
    树结构的实现和具体方法
    """

    def tree(self):
        """
        树结构的初始化
        """
        return MyDefaultDict(self.tree)

    @staticmethod
    def __node_add__(t, keys):
        """
        节点添加
        """
        for key in keys:
            t = t[key]
            t.num += 1

    @staticmethod
    def __filter__(t, items):
        """
        过滤节点
        """
        for index, key in enumerate(items):
            t = t[key]
            if t.num <= 1:
                items[index] = 'oo'
        return '/'.join(filter(lambda x: True if x != 'oo' else False, items))


class Metaclass(type):

    def __new__(cls, name, bases, attrs):
        count = 0
        attrs['__ConvertFunc__'] = []
        attrs['__ConvertFuncLabel__'] = []
        for k, v in attrs.items():
            if 'convert' in k:
                attrs['__ConvertFunc__'].append(k)
                attrs['__ConvertFuncLabel__'].append(k[2:].replace('_convert__', ''))
                count += 1
        attrs['__ConvertFuncCount__'] = count
        return type.__new__(cls, name, bases, attrs)


class Decorator:

    @staticmethod
    def exc(func):

        def wrapper(*args):
            try:
                return func(*args)
            except:
                return None

        return wrapper



#!/usr/bin/python3
# -*- coding:utf-8 -*-


class FieldHandler:
    """
    适用结构：字典列表嵌套结构
    """

    @classmethod
    def update_field_value(cls, val, **fields):
        """
        修改字段值
        :param val:
        :param fields:
        :return:
        """
        if isinstance(val, dict):
            for field, v in fields.items():
                if field in val:
                    val[field] = v

            for key in val:
                cls.update_field_value(val[key], **fields)

        if isinstance(val, list):
            for item in val:
                cls.update_field_value(item, **fields)

    @classmethod
    def update_field_name(cls, val, **fields):
        """
        修改字段名
        :param val:
        :param fields:
        :return:
        """
        if isinstance(val, dict):
            for field, v in fields.items():
                if field in val:
                    val[v] = val.pop(field)

            for key in val:
                cls.update_field_name(val[key], **fields)

        if isinstance(val, list):
            for item in val:
                cls.update_field_name(item, **fields)

    @classmethod
    def remove_field(cls, val, *fields):
        """
        删除字段（递归）
        @param val:
        @param fields:
        @return:
        """
        if isinstance(val, dict):
            for field in fields:
                val.pop(field, "404")

            for key in val:
                cls.remove_field(val[key], *fields)

        if isinstance(val, list):
            for item in val:
                cls.remove_field(item, *fields)

    @classmethod
    def add_field(cls, val, **fields):
        """
        添加字段,只针对列表
        :param val:
        :param fields:
        :return:
        """

        if isinstance(val, list):
            for item in val:
                for field, v in fields.items():
                    item[field] = v
